# -*- coding: utf-8 -*-
from django.core.management import BaseCommand
from ...utils import run


class Command(BaseCommand):
    help = 'Download and save bank transactions.'

    def handle(self, *args, **options):
        run()



# -*- coding: utf-8 -*-
import datetime
import logging
from django.db import IntegrityError
from django.db import transaction as db_transaction
from requests import HTTPError
from fiobank import FioBank
from models import Transaction, Account

logger = logging.getLogger(__name__)
if not logger.handlers:
    logger.addHandler(logging.StreamHandler())


@db_transaction.commit_manually
def run():
    try:
        account_list = Account.objects.all()
        for account in account_list:
            try:
                last_transaction = Transaction.objects.get_last_transaction(
                    account.id)

                client = FioBank(token=account.token)

                if last_transaction:
                    lookup_params = {'from_id': last_transaction.transaction_id}
                    transaction_list = client.last(**lookup_params)
                else:
                    from_date = '{0}-01-01'.format(datetime.date.today().year)
                    transaction_list = client.last(from_date=from_date)

            except HTTPError, e:
                logger.error('Error in process account {0}'.format(account))
                logger.exception(e)
                break
            else:
                for data in transaction_list:
                    print data
                    sid = db_transaction.savepoint()
                    try:
                        bank_trans = Transaction()
                        bank_trans.account = account
                        bank_trans.assign(data)
                        bank_trans.save()
                    except IntegrityError:
                        db_transaction.savepoint_rollback(sid)  # record exist
                    else:
                        db_transaction.savepoint_commit(sid)
    except Exception, e:
        db_transaction.rollback()
        raise e
    finally:
        db_transaction.commit()



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _

__all__ = ('TransactionManager', 'Account', 'Transaction')


class TransactionManager(models.Manager):

    def get_last_transaction(self, account_id):
        data = self.get_query_set().filter(account_id=account_id).order_by(
            '-transaction_id')[:1]
        if data:
            return data[0]
        else:
            return None


class Account(models.Model):
    name = models.CharField(_('Name'), max_length=255)
    account_number = models.CharField(_('Account number'), max_length=16)
    bank_code = models.CharField(_('Bank code'), max_length=10)
    token = models.CharField(_('Token'), max_length=64)

    class Meta:
        verbose_name = _('Account')
        verbose_name_plural = _('Accounts')
        unique_together = ('account_number', 'bank_code')

    def __unicode__(self):
        return self.name


class Transaction(models.Model):
    account = models.ForeignKey(Account)
    transaction_id = models.BigIntegerField(_('Transaction id'))
    date = models.DateField(_('Date'))
    amount = models.FloatField(_('Amount'))
    currency = models.CharField(_('Currency'), max_length=3)
    account_number = models.CharField(_('Account number'), max_length=16)
    account_name = models.CharField(_('Account name'), max_length=255,
                                    blank=True, null=True)
    bank_code = models.CharField(_('Bank code'), max_length=10)
    bic = models.CharField(_('Bic'), max_length=11, blank=True, null=True)
    bank_name = models.CharField(_('Bank name'), max_length=255)
    constant_symbol = models.CharField(_('Constant symbol'), max_length=4,
                                       blank=True, null=True)
    variable_symbol = models.CharField(_('Variable symbol'), max_length=10,
                                       blank=True, null=True)
    specific_symbol = models.CharField(_('Specific symbol'), max_length=10,
                                       blank=True, null=True)
    user_identification = models.CharField(_('User identification'),
                                           max_length=50, blank=True,
                                           null=True)
    recipient_message = models.CharField(_('Recipient message'),
                                         max_length=140, blank=True,
                                         null=True)
    type = models.CharField(_('Type'), max_length=255)
    executor = models.CharField(_('Executor'), max_length=255, blank=True,
                                null=True)
    specification = models.CharField(_('Specification'), max_length=255,
                                     blank=True, null=True)
    comment = models.CharField(_('Comment'), max_length=255, blank=True,
                               null=True)
    instruction_id = models.CharField(_('Instruction id'), max_length=15)

    objects = TransactionManager()

    class Meta:
        verbose_name = _('Transaction')
        verbose_name_plural = _('Transactions')
        unique_together = ('instruction_id',)

    def __unicode__(self):
        return u'{0}'.format(self.transaction_id)

    def assign(self, transaction_data):
        ''' Assign data from dict

        :param transaction_data: Transaction data
        :type transaction_data: dict
        '''
        for property_name, value in transaction_data.items():
            if hasattr(self, property_name):
                self.__setattr__(property_name, value)

    @property
    def account_full(self):
        return '{0}/{1}'.format(self.account_number, self.bank_code)



# -*- coding: utf-8 -*-
import os
import sys
import shlex
import subprocess
import django_fiobank

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages  # NOQA

version = django_fiobank.__versionstr__


# release a version, publish to GitHub and PyPI
if sys.argv[-1] == 'publish':
    command = lambda cmd: subprocess.check_call(shlex.split(cmd))
    command('git tag v' + version)
    command('git push --tags origin master:master')
    command('python setup.py sdist upload')
    sys.exit()

setup(
    name='django_fiobank',
    version=version,
    description=('Django-app for managing and processing Fio Bank '
                 'transaction'),
    long_description=open('README.rst').read(),
    author='Martin Voldrich',
    author_email='rbas.cz@gmail.com',
    url='https://github.com/rbas/django-fiobank',
    packages=find_packages(),
    license=open('LICENSE').read(),
    install_requires=['fiobank>=0.0.3,<0.1', 'south'],
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet',
    )
)



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from .models import Account, Transaction


class AmountFilter(SimpleListFilter):
    title = _('Transaction type')

    parameter_name = 'transaction_type'

    def lookups(self, request, model_admin):
        return (
            ('all', _('All')),
            ('income', _('Income')),
            ('issue', _('Issue')),
        )

    def choices(self, cl):
        for lookup, title in self.lookup_choices:
            yield {
                'selected': self.value() == lookup,
                'query_string': cl.get_query_string({
                                                    self.parameter_name: lookup,
                                                    }, []),
                'display': title,
            }

    def queryset(self, request, queryset):
        if self.value() == 'income':
            return queryset.filter(amount__gt=0)
        elif self.value() == 'issue':
            return queryset.filter(amount__lt=0)
        elif self.value() == 'all':
            return queryset


class TransactionAdmin(admin.ModelAdmin):
    list_display = (
        'transaction_id', 'date', 'account_full', 'amount', 'currency',
        'bank_name', 'constant_symbol', 'variable_symbol',
        'specific_symbol', 'type',
        'executor', 'comment')
    list_filter = ('account__name', AmountFilter,)
    search_fields = ('account_number',)
    date_hierarchy = 'date'


admin.site.register(Account)
admin.site.register(Transaction, TransactionAdmin)
